@@ -55,6 +55,7 @@
 #include <ofono/gprs-context.h>
 #include <ofono/radio-settings.h>
 #include <ofono/audio-settings.h>
+#include <ofono/gnss.h>
 #include <ofono/stk.h>
 #include <ofono/ctm.h>
 #include <ofono/log.h>
@@ -308,6 +309,10 @@ static void xgendata_query(gboolean ok, GAtResult *result, gpointer user_data)
 
 	DBG("\n%s", gendata);
 
+	/* switch to GSM character set instead of IRA */
+	g_at_chat_send(data->dlcs[AUX_DLC], "AT+CSCS=\"GSM\"", none_prefix,
+							NULL, NULL, NULL);
+
 	/* disable UART for power saving */
 	g_at_chat_send(data->dlcs[AUX_DLC], "AT+XPOW=0,0,0", none_prefix,
 							NULL, NULL, NULL);
@@ -693,6 +698,8 @@ static void ifx_post_online(struct ofono_modem *modem)
 	ofono_cbs_create(modem, 0, "atmodem", data->dlcs[AUX_DLC]);
 	ofono_ussd_create(modem, 0, "atmodem", data->dlcs[AUX_DLC]);
 
+	ofono_gnss_create(modem, 0, "atmodem", data->dlcs[AUX_DLC]);
+
 	ofono_call_settings_create(modem, 0, "atmodem", data->dlcs[AUX_DLC]);
 	ofono_call_meter_create(modem, 0, "atmodem", data->dlcs[AUX_DLC]);
 	ofono_call_barring_create(modem, 0, "atmodem", data->dlcs[AUX_DLC]);
