@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -105,17 +105,11 @@ void sim_fs_free(struct sim_fs *fs)
 	if (fs->op_q) {
 		g_queue_foreach(fs->op_q, (GFunc) sim_fs_op_free, NULL);
 		g_queue_free(fs->op_q);
+		fs->op_q = NULL;
 	}
 
-	if (fs->contexts != NULL) {
-		GSList *l;
-
-		for (l = fs->contexts; l; l = l->next) {
-			struct ofono_sim_context *context = l->data;
-
-			sim_fs_context_free(context);
-		}
-	}
+	while (fs->contexts)
+		sim_fs_context_free(fs->contexts->data);
 
 	g_free(fs);
 }
@@ -907,8 +901,10 @@ int sim_fs_read(struct ofono_sim_context *context, int id,
 	if (fs->driver == NULL)
 		return -EINVAL;
 
-	if (fs->driver->read_file_info == NULL)
+	if (fs->driver->read_file_info == NULL) {
+		cb(0, 0, 0, NULL, 0, data);
 		return -ENOSYS;
+	}
 
 	if (fs->op_q == NULL)
 		fs->op_q = g_queue_new();
