@@ -515,6 +515,9 @@ static gboolean stk_alpha_id_set(struct ofono_stk *stk,
 	if (alpha == NULL)
 		return FALSE;
 
+	if (stk->current_agent == NULL)
+		return FALSE;
+
 	if (stk->respond_on_exit)
 		stk_agent_display_action(stk->current_agent, alpha, icon,
 						user_termination_cb, stk, NULL);
@@ -1519,10 +1522,8 @@ static gboolean handle_command_get_inkey(const struct stk_command *cmd,
 	gboolean alphabet = (qualifier & (1 << 0)) != 0;
 	gboolean ucs2 = (qualifier & (1 << 1)) != 0;
 	gboolean yesno = (qualifier & (1 << 2)) != 0;
-	/*
-	 * Note: immediate response and help parameter values are not
-	 * provided by current api.
-	 */
+	gboolean immediate = (qualifier & (1 << 3)) != 0;
+	/* Note: help parameter value is not provided by current api. */
 	int err;
 
 	if (text == NULL) {
@@ -1545,6 +1546,11 @@ static gboolean handle_command_get_inkey(const struct stk_command *cmd,
 						&gi->icon_id, ucs2,
 						request_key_cb, stk, NULL,
 						timeout);
+	else if (immediate)
+		err = stk_agent_request_quick_digit(stk->current_agent,
+							text, &gi->icon_id,
+							request_key_cb, stk,
+							NULL, timeout);
 	else
 		err = stk_agent_request_digit(stk->current_agent, text,
 						&gi->icon_id, request_key_cb,
@@ -2021,11 +2027,6 @@ static gboolean handle_command_send_ussd(const struct stk_command *cmd,
 					cmd->send_ussd.ussd_string.len,
 					send_ussd_callback, stk);
 
-	if (err >= 0) {
-		stk->cancel_cmd = send_ussd_cancel;
-
-		return FALSE;
-	}
 
 	if (err == -ENOSYS) {
 		rsp->result.type = STK_RESULT_TYPE_NOT_CAPABLE;
@@ -2038,6 +2039,20 @@ static gboolean handle_command_send_ussd(const struct stk_command *cmd,
 		return TRUE;
 	}
 
+	if (err < 0) {
+		unsigned char no_cause_result[] = { 0x00 };
+
+		/*
+		 * We most likely got an out of memory error, tell SIM
+		 * to retry
+		 */
+		ADD_ERROR_RESULT(rsp->result, STK_RESULT_TYPE_TERMINAL_BUSY,
+					no_cause_result);
+		return TRUE;
+	}
+
+	stk->cancel_cmd = send_ussd_cancel;
+
 	stk_alpha_id_set(stk, cmd->send_ussd.alpha_id,
 				&cmd->send_ussd.text_attr,
 				&cmd->send_ussd.icon_id);
@@ -2631,6 +2646,9 @@ static gboolean handle_setup_call_confirmation_req(struct stk_command *cmd,
 	if (alpha_id == NULL)
 		goto out;
 
+	if (stk->current_agent == FALSE)
+		goto out;
+
 	err = stk_agent_confirm_call(stk->current_agent, alpha_id,
 					&sc->icon_id_usr_cfm,
 					confirm_handled_call_cb,
@@ -2738,6 +2756,9 @@ void ofono_stk_proactive_command_notify(struct ofono_stk *stk,
 		case STK_COMMAND_TYPE_GET_INPUT:
 		case STK_COMMAND_TYPE_PLAY_TONE:
 		case STK_COMMAND_TYPE_SETUP_CALL:
+		case STK_COMMAND_TYPE_SEND_SMS:
+		case STK_COMMAND_TYPE_SEND_USSD:
+		case STK_COMMAND_TYPE_SEND_DTMF:
 			send_simple_response(stk, STK_RESULT_TYPE_NOT_CAPABLE);
 			return;
 
